@@ -1,11 +1,13 @@
 import logging
-import typing
 from collections import defaultdict
+from collections import namedtuple
+from datetime import datetime
 from typing import Dict
 from typing import List
 from typing import Optional
 from typing import Union
 
+import pytz
 from OpenSSL import SSL
 from rx.subjects import Subject
 from twisted.internet import reactor
@@ -56,12 +58,304 @@ class NoVortexException(Exception):
     """
 
 
+_ConnectionDateSuccess = namedtuple(
+    "_ConnectionDateSuccess", ("dateTime", "success", "ip")
+)
+
+
+class _VortexConnectionRateLimit:
+    _ROLLING_WINDOW_SECONDS = 10.0
+    _NEW_CONNECTIONS_IN_WINDOW = 5
+
+    def __init__(self):
+        self._connections: list[_ConnectionDateSuccess] = []
+        self._peerConnectionLimitPerIp = None
+
+    def _hasHitPeerConnectionLimit(
+        self, connectionCountForPeer: int, fromIp: str
+    ) -> bool:
+        if not self._peerConnectionLimitPerIp:
+            return True
+
+        if connectionCountForPeer + 1 <= self._peerConnectionLimitPerIp:
+            return True
+
+        logger.info(
+            "Closing new connection from %s,"
+            " this peer has %s connections, our limit is %s",
+            fromIp,
+            connectionCountForPeer,
+            self._peerConnectionLimitPerIp,
+        )
+
+        return False
+
+    def canConnect(
+        self,
+        connectionCountForPeer: int,
+        totalConnectionCount: int,
+        fromIp: str,
+    ) -> bool:
+        if not self._hasHitPeerConnectionLimit(connectionCountForPeer, fromIp):
+            return False
+
+        now = datetime.now(pytz.utc)
+        self._connections = list(
+            filter(
+                lambda item: (now - item.dateTime).total_seconds()
+                < self._ROLLING_WINDOW_SECONDS,
+                self._connections,
+            )
+        )
+
+        connectedInWindow = len(
+            list(filter(lambda item: item.success, self._connections))
+        )
+
+        if connectedInWindow < self._NEW_CONNECTIONS_IN_WINDOW:
+            self._connections.append(_ConnectionDateSuccess(now, True, fromIp))
+            return True
+
+        self._connections.append(_ConnectionDateSuccess(now, False, fromIp))
+
+        earliestDate = min([c.dateTime for c in self._connections])
+        logger.info(
+            "Closing new connection from %s,"
+            " we've allowed %s"
+            " and closed %s connections in the last %s seconds,"
+            " we have %s connections",
+            fromIp,
+            connectedInWindow,
+            len(self._connections) - connectedInWindow,
+            now - earliestDate,
+            totalConnectionCount,
+        )
+
+        return False
+
+    def setPeerConnectionLimitPerIp(self, limit):
+        self._peerConnectionLimitPerIp = limit
+
+
 VortexUuidList = List[str]
 
 
+class _VortexFactoryVortexInfo:
+    def __init__(self):
+        self.__vortexServersByName: Dict[str, List[VortexServer]] = defaultdict(
+            list
+        )
+        self.__vortexClientsByName: Dict[str, List[VortexABC]] = defaultdict(
+            list
+        )
+
+        self.__vortexServers: tuple[VortexServer] = tuple()
+        self.__vortexClients: tuple[VortexABC] = tuple()
+        self.__allVortexes: tuple[VortexABC] = tuple()
+
+        self.__remoteVortexUuidsByName = defaultdict(set)
+        self.__remoteVortexNameByUuid = {}
+
+        self.__remoteVortexUuids: set[str] = set()
+        self.__inboundConnectionCount: int = 0
+        self.__inboundConnectionCountPerIp = defaultdict(lambda: 0)
+
+        self.__localVortexUuidsByName = defaultdict(set)
+        self.__localVortexNameByUuid = {}
+
+    def addVortexServer(self, vortexName: str, vortex: VortexServer):
+        vortexes = self.__vortexServersByName[vortexName]
+
+        if vortex in vortexes:
+            raise Exception("Vortex is already registered, %s", vortex)
+
+        vortexes.append(vortex)
+        self.rebuildStructs()
+
+    def addVortexClient(self, vortexName: str, vortex: VortexABC):
+        vortexes = self.__vortexClientsByName[vortexName]
+
+        if vortex in vortexes:
+            raise Exception("Vortex is already registered, %s", vortex)
+
+        vortexes.append(vortex)
+        self.rebuildStructs()
+
+    def rebuildStructs(self):
+
+        vortexServers = []
+        for vortexes in self.__vortexServersByName.values():
+            vortexServers.extend(vortexes)
+
+        self.__vortexServers = tuple(vortexServers)
+
+        vortexClients = []
+        for vortexes in self.__vortexClientsByName.values():
+            vortexClients.extend(vortexes)
+
+        self.__vortexClients = tuple(vortexClients)
+
+        self.__allVortexes = self.__vortexServers + self.__vortexClients
+
+        self.__remoteVortexUuidsByName: dict[
+            str, set[tuple[VortexABC, str]]
+        ] = defaultdict(set)
+
+        self.__remoteVortexNameByUuid: dict[str, tuple[VortexABC, str]] = {}
+
+        self.__localVortexUuidsByName: dict[
+            str, set[tuple[VortexABC, str]]
+        ] = defaultdict(set)
+
+        self.__localVortexNameByUuid: dict[str, tuple[VortexABC, str]] = {}
+
+        self.__inboundConnectionCount: int = 0
+        self.__inboundConnectionCountPerIp = defaultdict(lambda: 0)
+        for vortex in self.__vortexServers:
+            self.__inboundConnectionCount += len(vortex.connections)
+            for conn in vortex.connections.values():
+                self.__inboundConnectionCountPerIp[conn.ip] += 1
+
+        for vortex in self.allVortexes:
+            for remoteVortexInfo in vortex.remoteVortexInfo:
+                self.__remoteVortexUuidsByName[remoteVortexInfo.name].add(
+                    (vortex, remoteVortexInfo.uuid)
+                )
+                assert (
+                    remoteVortexInfo.uuid not in self.__remoteVortexNameByUuid
+                ), "Remote UUID is already registered - all vortexes %s" % str(
+                    self.allVortexes
+                )
+                self.__remoteVortexNameByUuid[remoteVortexInfo.uuid] = (
+                    vortex,
+                    remoteVortexInfo.name,
+                )
+
+            self.__localVortexUuidsByName[vortex.localVortexInfo.name].add(
+                (vortex, vortex.localVortexInfo.uuid)
+            )
+            assert (
+                vortex.localVortexInfo.uuid not in self.__localVortexNameByUuid
+            ), "Local UUID is already registered - all vortexes %s" % str(
+                self.allVortexes
+            )
+            self.__localVortexNameByUuid[vortex.localVortexInfo.uuid] = (
+                vortex,
+                vortex.localVortexInfo.name,
+            )
+
+        self.__remoteVortexUuids = set(self.__remoteVortexNameByUuid)
+
+    def getRemoteVortexInfos(
+        self, name=None, uuid=None
+    ) -> list[tuple[VortexABC, str]]:
+        assert name or uuid, "We expect a name or uuid"
+
+        # If we're passed a UUID, then there will only be one result
+        if uuid:
+            vortex, foundName = self.__remoteVortexNameByUuid.get(
+                uuid, (None, None)
+            )
+            if not foundName:
+                return []
+            return [(vortex, uuid)]
+
+        # Name is asserted
+        return [
+            (vortex, foundUuid)
+            for vortex, foundUuid in self.__remoteVortexUuidsByName.get(
+                name, []
+            )
+        ]
+
+    def getRemoteVortexInfoByIp(self, ip: str, vortexName=None) -> str:
+        # Ignore the port if any
+        ip = ip.split(":")[0]
+
+        from vortex.VortexConnectionABC import VortexConnectionABC
+
+        targetConns: list[VortexConnectionABC] = []
+        for server in self.vortexSevers:
+            for conn in server.connections.values():
+                if (
+                    not conn.closed
+                    and not conn.timedOut
+                    and conn.ip == ip
+                    and (
+                        conn.remoteVortexName == vortexName
+                        or vortexName is None
+                    )
+                ):
+                    targetConns.append(conn)
+
+        if len(targetConns) > 1:
+            logger.warning(
+                "Multiple vortexes found for ip=%s, name=%s,"
+                " choosing latest connection",
+                ip,
+                vortexName,
+            )
+            targetConns.sort(key=lambda c: c.connectDateTime)
+            # As of Python3.6, dict are ordered, so the newest is the last.
+            return targetConns[-1].remoteVortexUuid
+
+        if not targetConns:
+            raise NoVortexException(f"Can not find vortex with IP {ip}")
+
+        return targetConns[0].remoteVortexUuid
+
+    @property
+    def getRemoteClientVortexInfos(self) -> List[VortexInfo]:
+        return [
+            VortexInfo(name=name, uuid=uuid)
+            for name, (vortex, uuid) in self.__remoteVortexUuidsByName.items()
+        ]
+
+    @property
+    def getRemoteVortexUuids(self) -> set[str]:
+        return self.__remoteVortexUuids
+
+    @property
+    def getRemoteVortexNames(self) -> list[str]:
+        return list(self.__remoteVortexUuidsByName)
+
+    @property
+    def getInboundConnectionCount(self) -> int:
+        return self.__inboundConnectionCount
+
+    def getInboundConnectionCountForPeer(self, peerIp: str) -> int:
+        return self.__inboundConnectionCountPerIp[peerIp]
+
+    def isVortexNameLocal(self, vortexName: str) -> bool:
+        return vortexName in self.__localVortexUuidsByName
+
+    def getLocalVortexClients(self, localVortexName: str) -> List[VortexABC]:
+        return [
+            vortex
+            for vortex, _ in self.__localVortexUuidsByName.get(
+                localVortexName, []
+            )
+        ]
+
+    def isRemoteVortexOnline(self, uuid):
+        return uuid in self.__remoteVortexNameByUuid
+
+    @property
+    def vortexSevers(self) -> tuple[VortexServer]:
+        return self.__vortexServers
+
+    @property
+    def vortexClients(self) -> tuple[VortexABC]:
+        return self.__vortexClients
+
+    @property
+    def allVortexes(self) -> tuple[VortexABC]:
+        return self.__allVortexes
+
+
 class VortexFactory:
-    __vortexServersByName: Dict[str, List[VortexABC]] = defaultdict(list)
-    __vortexClientsByName: Dict[str, List[VortexABC]] = defaultdict(list)
+    __vortexInfoState = _VortexFactoryVortexInfo()
+    __connectionRateLimit = _VortexConnectionRateLimit()
 
     __vortexStatusChangeSubjectsByName: Dict[str, Subject] = {}
 
@@ -69,6 +363,8 @@ class VortexFactory:
 
     __listeningPorts = []
 
+    _DEBUG_LOGGING = False
+
     def __init__(self):
         raise Exception("Vortex Factory should not be instantiated")
 
@@ -76,57 +372,20 @@ class VortexFactory:
     @inlineCallbacks
     def shutdown(cls):
         cls.__isShutdown = True
-        for vortex in VortexFactory._allVortexes():
+        for vortex in cls.__vortexInfoState.allVortexes:
             if hasattr(vortex, "close"):
                 vortex.close()
                 yield deferLater(reactor, 0.05, lambda: None)
+
         while cls.__listeningPorts:
             cls.__listeningPorts.pop().stopListening()
             yield deferLater(reactor, 0.05, lambda: None)
 
-    @classmethod
-    def _getVortexSendRefs(
-        cls, name=None, uuid=None
-    ) -> List[typing.Tuple[VortexABC, List[str]]]:
-        assert name or uuid
-
-        results = []
-
-        def iterVortexes(vortexes):
-            for vortex in vortexes:
-                uuids: List[str] = []
-                # logger.debug("FROM : %s", vortex.localVortexInfo)
-                for remoteVortexInfo in vortex.remoteVortexInfo:
-                    # logger.debug("        REMOTE : %s", remoteVortexInfo)
-                    if (name is None or remoteVortexInfo.name == name) and (
-                        uuid is None or remoteVortexInfo.uuid == uuid
-                    ):
-                        uuids.append(remoteVortexInfo.uuid)
-
-                if uuids:
-                    results.append((vortex, uuids))
-
-        for vortexList in cls.__vortexServersByName.values():
-            iterVortexes(vortexList)
-
-        for vortexList in cls.__vortexClientsByName.values():
-            iterVortexes(vortexList)
-
-        return results
+        cls.__vortexInfoState = _VortexFactoryVortexInfo()
 
     @classmethod
-    def _allVortexes(cls):
-        """All Vortexes
-
-        :return: A list of all the vortexes, both client and server
-        """
-        vortexes = []
-        for vortexList in cls.__vortexServersByName.values():
-            vortexes += vortexList
-        for vortexList in cls.__vortexClientsByName.values():
-            vortexes += vortexList
-
-        return vortexes
+    def setPeerConnectionLimitPerIp(cls, limit: int):
+        cls.__connectionRateLimit.setPeerConnectionLimitPerIp(limit)
 
     @classmethod
     def createServer(cls, name: str, rootResource) -> None:
@@ -144,7 +403,7 @@ class VortexFactory:
         """
 
         vortexServer = VortexServer(name)
-        cls.__vortexServersByName[name].append(vortexServer)
+        cls.__vortexInfoState.addVortexServer(name, vortexServer)
 
         vortexResource = VortexServerHttpResource(vortexServer)
         rootResource.putChild(b"vortex", vortexResource)
@@ -200,7 +459,7 @@ class VortexFactory:
         )
 
         vortexServer = VortexServer(name, requiresBase64Encoding=False)
-        cls.__vortexServersByName[name].append(vortexServer)
+        cls.__vortexInfoState.addVortexServer(name, vortexServer)
         vortexWebsocketServerFactory = VortexWebsocketServerFactory(
             vortexServer
         )
@@ -240,7 +499,7 @@ class VortexFactory:
         cls, name: str
     ) -> VortexWebSocketUpgradeResource:
         vortexServer = VortexServer(name, requiresBase64Encoding=False)
-        cls.__vortexServersByName[name].append(vortexServer)
+        cls.__vortexInfoState.addVortexServer(name, vortexServer)
 
         vortexWebsocketServerFactory = VortexWebsocketServerFactory(
             vortexServer
@@ -266,7 +525,7 @@ class VortexFactory:
         """
 
         vortexServer = VortexServer(name, requiresBase64Encoding=False)
-        cls.__vortexServersByName[name].append(vortexServer)
+        cls.__vortexInfoState.addVortexServer(name, vortexServer)
         vortexWebsocketServerFactory = VortexWebsocketServerFactory(
             vortexServer
         )
@@ -292,7 +551,7 @@ class VortexFactory:
         """
 
         vortexServer = VortexServer(name)
-        cls.__vortexServersByName[name].append(vortexServer)
+        cls.__vortexInfoState.addVortexServer(name, vortexServer)
         vortexTcpServerFactory = VortexTcpServerFactory(vortexServer)
         port = reactor.listenTCP(port, vortexTcpServerFactory)
         cls.__listeningPorts.append(port)
@@ -313,7 +572,7 @@ class VortexFactory:
         logger.info("Connecting to Peek Server HTTP %s:%s", host, port)
 
         vortexClient = VortexClientHttp(name)
-        cls.__vortexClientsByName[name].append(vortexClient)
+        cls.__vortexInfoState.addVortexClient(name, vortexClient)
 
         return vortexClient.connect(host, port)
 
@@ -364,7 +623,9 @@ class VortexFactory:
         vortexWebsocketClientFactory = VortexClientWebsocketFactory(
             name, url=url
         )
-        cls.__vortexClientsByName[name].append(vortexWebsocketClientFactory)
+        cls.__vortexInfoState.addVortexClient(
+            name, vortexWebsocketClientFactory
+        )
 
         if vortexWebsocketClientFactory.isSecure and sslEnableMutualTLS:
             trustedCertificateAuthorities = parseTrustRootFromBundle(
@@ -415,106 +676,59 @@ class VortexFactory:
         logger.info("Connecting to Peek Server TCP %s:%s", host, port)
 
         vortexClient = VortexClientTcp(name)
-        cls.__vortexClientsByName[name].append(vortexClient)
+        cls.__vortexInfoState.addVortexClient(name, vortexClient)
 
         return vortexClient.connect(host, port)
 
     @classmethod
     def addCustomServerVortex(cls, vortex: VortexABC):
-        cls.__vortexServersByName[vortex.localVortexInfo.name].append(vortex)
+        cls.__vortexInfoState.addVortexServer(
+            vortex.localVortexInfo.name, vortex
+        )
 
     @classmethod
-    def isVortexNameLocal(cls, vortexName: str) -> bool:
-        for vortex in cls._allVortexes():
-            if vortex.localVortexInfo.name == vortexName:
-                return True
-
-        return False
+    def canConnect(cls, fromIp: str) -> bool:
+        return cls.__connectionRateLimit.canConnect(
+            cls.__vortexInfoState.getInboundConnectionCountForPeer(fromIp),
+            cls.__vortexInfoState.getInboundConnectionCount,
+            fromIp,
+        )
 
     @classmethod
     def isVortexUuidOnline(cls, vortexUuid: str) -> bool:
-        return bool(cls._getVortexSendRefs(uuid=vortexUuid))
+        return cls.__vortexInfoState.isRemoteVortexOnline(uuid=vortexUuid)
+
+    @classmethod
+    def isVortexNameLocal(cls, vortexName: str) -> bool:
+        return cls.__vortexInfoState.isVortexNameLocal(vortexName=vortexName)
 
     @classmethod
     def getLocalVortexClients(cls, localVortexName: str) -> List[VortexABC]:
-        vortexes: List[VortexABC] = []
-
-        for items in cls.__vortexClientsByName.values():
-            vortexes.extend(
-                filter(
-                    lambda x: x.localVortexInfo.name == localVortexName, items
-                )
-            )
-
-        return vortexes
+        return cls.__vortexInfoState.getLocalVortexClients(
+            localVortexName=localVortexName
+        )
 
     @classmethod
     def getRemoteClientVortexInfos(cls) -> List[VortexInfo]:
-        clientVortexInfos = []
-
-        for vortexes in cls.__vortexServersByName.values():
-            for vortex in vortexes:
-                clientVortexInfos.extend(vortex.remoteVortexInfo)
-
-        return clientVortexInfos
+        return cls.__vortexInfoState.getRemoteClientVortexInfos
 
     @classmethod
-    def getRemoteVortexUuids(cls) -> List[str]:
-        remoteUuids = []
-
-        for vortex in cls._allVortexes():
-            for remoteVortexInfo in vortex.remoteVortexInfo:
-                remoteUuids.append(remoteVortexInfo.uuid)
-
-        return remoteUuids
+    def getRemoteVortexUuids(cls) -> set[str]:
+        return cls.__vortexInfoState.getRemoteVortexUuids
 
     @classmethod
-    def getRemoteVortexName(cls) -> List[str]:
-        remoteNames = set()
+    def getInboundConnectionCount(cls) -> int:
+        return cls.__vortexInfoState.getInboundConnectionCount
 
-        for vortex in cls._allVortexes():
-            for remoteVortexInfo in vortex.remoteVortexInfo:
-                remoteNames.add(remoteVortexInfo.name)
-
-        return list(remoteNames)
+    @classmethod
+    def getRemoteVortexName(cls) -> list[str]:
+        return cls.__vortexInfoState.getRemoteVortexNames
 
     @classmethod
     def getRemoteVortexInfoByIp(cls, ip: str, vortexName=None) -> str:
-        # Ignore the port if any
-        ip = ip.split(":")[0]
-
-        from vortex.VortexConnectionABC import VortexConnectionABC
-
-        targetConns: list[VortexConnectionABC] = []
-        for servers in cls.__vortexServersByName.values():
-            for server in servers:
-                for conn in server.connections.values():
-                    if (
-                        not conn.closed
-                        and not conn.timedOut
-                        and conn.ip == ip
-                        and (
-                            conn.remoteVortexName == vortexName
-                            or vortexName is None
-                        )
-                    ):
-                        targetConns.append(conn)
-
-        if len(targetConns) > 1:
-            logger.warning(
-                "Multiple vortexes found for ip=%s, name=%s,"
-                " choosing latest connection",
-                ip,
-                vortexName,
-            )
-            targetConns.sort(key=lambda c: c.connectDateTime)
-            # As of Python3.6, dict are ordered, so the newest is the last.
-            return targetConns[-1].remoteVortexUuid
-
-        if not targetConns:
-            raise NoVortexException(f"Can not find vortex with IP {ip}")
-
-        return targetConns[0].remoteVortexUuid
+        return cls.__vortexInfoState.getRemoteVortexInfoByIp(
+            ip=ip, vortexName=vortexName
+        )
 
     @classmethod
     @inlineCallbacks
@@ -541,8 +755,9 @@ class VortexFactory:
 
         :return: A C{Deferred} which will callback when the message has been sent.
         """
-
-        vortexAndUuids = cls._getVortexSendRefs(destVortexName, destVortexUuid)
+        vortexAndUuids = cls.__vortexInfoState.getRemoteVortexInfos(
+            name=destVortexName, uuid=destVortexUuid
+        )
         if not vortexAndUuids:
             raise NoVortexException(
                 "Can not find vortexes to send message to,"
@@ -563,14 +778,13 @@ class VortexFactory:
                     ] = yield PayloadEnvelope.base64EncodeDefer(vortexMsg)
 
         deferreds = []
-        for vortex, uuids in vortexAndUuids:
-            for uuid in uuids:
-                msgToUse = (
-                    base64VortexMsgs
-                    if vortex.requiresBase64Encoding
-                    else vortexMsgs
-                )
-                deferreds.append(vortex.sendVortexMsg(msgToUse, uuid))
+        for vortex, uuid in vortexAndUuids:
+            msgToUse = (
+                base64VortexMsgs
+                if vortex.requiresBase64Encoding
+                else vortexMsgs
+            )
+            deferreds.append(vortex.sendVortexMsg(msgToUse, uuid))
 
         results = yield DeferredList(deferreds)
         return results
@@ -622,6 +836,10 @@ class VortexFactory:
 
         return DeferredList(deferreds)
 
+    @classmethod
+    def connectionChanged(cls):
+        cls.__vortexInfoState.rebuildStructs()
+
     @classmethod
     def subscribeToVortexStatusChange(cls, vortexName: str) -> Subject:
         """Subscribe to Vortex Status Change
@@ -650,3 +868,5 @@ class VortexFactory:
 
         if vortexName in cls.__vortexStatusChangeSubjectsByName:
             cls.__vortexStatusChangeSubjectsByName[vortexName].on_next(online)
+
+        cls.connectionChanged()
