@@ -1,13 +1,16 @@
 import logging
-from abc import abstractmethod, ABCMeta
+from abc import ABCMeta
+from abc import abstractmethod
 from collections import namedtuple
-from datetime import datetime
-from typing import Union, Optional, List
+from typing import List
+from typing import Optional
+from typing import Union
 
-import pytz
 from twisted.internet import reactor
-from twisted.internet.defer import DeferredSemaphore
-from twisted.internet.defer import fail, Deferred, succeed, inlineCallbacks
+from twisted.internet.defer import Deferred
+from twisted.internet.defer import fail
+from twisted.internet.defer import inlineCallbacks
+from twisted.internet.defer import succeed
 from twisted.python import failure
 
 from vortex.DeferUtil import vortexLogFailure
@@ -16,6 +19,7 @@ from vortex.PayloadEnvelope import PayloadEnvelope
 from vortex.TupleSelector import TupleSelector
 from vortex.VortexABC import SendVortexMsgResponseCallable
 from vortex.VortexFactory import VortexFactory
+from vortex.VortexUtil import limitConcurrency, _dedupProcessorCallKeys
 
 logger = logging.getLogger(__name__)
 
@@ -64,11 +68,6 @@ class _ObserverData:
 
 
 class TupleDataObservableHandler:
-    # To avoid saturating the CPU, we will prepare only 10 responses at a time.
-    # Once the responses are prepared,
-    #  they will be queued in the VortexPushProducer
-    _prepareResponsesSemaphore = DeferredSemaphore(10)
-
     def __init__(
         self,
         observableName,
@@ -96,7 +95,7 @@ class TupleDataObservableHandler:
 
         self._endpoint = PayloadEndpoint(
             self._filt,
-            self._processBeforeSemaphore,
+            self._processLimited,
             acceptOnlyFromVortex=acceptOnlyFromVortex,
         )
 
@@ -165,45 +164,16 @@ class TupleDataObservableHandler:
             raise Exception(msg)
         return vortexMsg
 
-    def _processBeforeSemaphore(self, *args, **kwargs):
-        d = self._prepareResponsesSemaphore.run(
-            self._processAfterSemaphore, datetime.now(pytz.utc), *args, **kwargs
-        )
-        d.addErrback(vortexLogFailure, logger, consumeError=True)
+    def _processLimited(self, *args, **kwargs):
+        d = self._processLimitedDecorated(*args, **kwargs)
+        if d:
+            d.addErrback(vortexLogFailure, logger, consumeError=True)
 
-    @inlineCallbacks
-    def _processAfterSemaphore(
-        self,
-        queuedTime: datetime,
-        payloadEnvelope: PayloadEnvelope,
-        *args,
-        **kwargs,
-    ):
-        deltaTime = datetime.now(pytz.utc) - queuedTime
-        queueSize = len(self._prepareResponsesSemaphore.waiting)
-
-        def logIt(logFunc):
-            logFunc(
-                "_process call was queued for %s, queue size " "%s",
-                deltaTime,
-                queueSize,
-            )
-
-        if 15.0 < deltaTime.total_seconds() or 2000 < queueSize:
-            logIt(logger.warning)
-
-        elif 5.0 < deltaTime.total_seconds() or 200 < queueSize:
-            logIt(logger.debug)
-
-        startTime = datetime.now(pytz.utc)
-        yield self._process(payloadEnvelope, *args, **kwargs)
-        processingTime = datetime.now(pytz.utc) - startTime
-        if 5.0 < processingTime.total_seconds():
-            logger.debug(
-                "Payload endpoint took %s\npayload.filt=%s",
-                processingTime,
-                payloadEnvelope.filt,
-            )
+    @limitConcurrency(
+        logger, 25, deduplicateBasedOnKwArgsKey=_dedupProcessorCallKeys
+    )
+    def _processLimitedDecorated(self, *args, **kwargs):
+        return self._process(*args, **kwargs)
 
     @inlineCallbacks
     def _process(
@@ -214,8 +184,15 @@ class TupleDataObservableHandler:
         **kwargs,
     ):
         if not VortexFactory.isVortexUuidOnline(vortexUuid):
+            logger.debug(
+                "Vortex %s is no longer online, skipping reply", vortexUuid
+            )
             return
 
+        # logger.debug(
+        #     "Vortex %s is STILL online, continuing to reply", vortexUuid
+        # )
+
         tupleSelector = payloadEnvelope.filt["tupleSelector"]
         tsStr = tupleSelector.toJsonStr()
 
